#!/bin/bash

rm -f cwmonitor_*

. `dirname $0`/simple_curses.sh

log_delay=60
deflt_log_lines=36
log_lines=$deflt_log_lines
log_updated=1
divider=2

function do_help()
{
cat << EOF
Usage: ${0##*/} [-h] [-l LINES]
Monitor of 'wildix-cw' process

-h          display this help and exit
-l LINES    number of lines in terminal window (default - $deflt_log_lines)
EOF
}

while [ "$#" -gt 0 ]; do
	case $1 in
		-h|--help)
			do_help
			exit
			;;
		-l|--lines)
			if [ "$#" -gt 1 ]; then
				log_lines=$2
				shift 2
				continue
			else
				echo 'ERROR: Must specify a lines count.' >&2
				exit 1
			fi
			;;
		*)
			break
	esac
	shift
done

if [ "$log_lines" -lt "20" ] || [ "$log_lines" -gt "80" ]; then
	log_lines=$deflt_log_lines
fi

t_point=`date +%s`                                           # time point
t_next=`expr $t_point + $log_delay`                          # time next
cwpid=`pidof callweaver`                                     # initial callweaver pid
cwlog=`mktemp cwstat_$cwpid-XXXXXX`
cwstat=`mktemp cw-load-statistic_$cwpid-XXXXXX`

for n in `seq 0 $log_lines` ; do echo >>$cwlog ; done        # initialize memory statistics buffer

record_line() {
	a_time=`date +%T`
	a_pid=`ps ax | grep 'callweaver .* -v -v -v' | grep -v grep | sed -e 's/^[[:blank:]]*//' -e 's/[[:blank:]].*$//' `
	a_mem=`cat /proc/$a_pid/status | grep ^VmRSS | sed -e 's/^[^[:blank:]]*[[:blank:]]*//' -e 's/[[:blank:]].*$//' -e "s/\n+//g" `

	z=`expr $a_mem / 1024`
	echo -ne $a_time "\t\t" $a_pid "\t\t" $z "M\t\t" >>$cwlog
	for n in `seq 0 $z` ; do echo -n '#' >>$cwlog ; done
	echo>>$cwlog
}

record_line

log_update() {
	t_point=`date +%s`
	if [ $t_next -lt $t_point ] ; then 
		record_line
		log_updated=1
		t_next=`expr $t_point + $log_delay`
	fi
}


main (){

    log_update

    #basic informations, hostname, date,...
    window "`hostname`" "red" "30%"
    append "`date`"
    addsep
    append_tabbed "Up since|`uptime | cut -f1 -d"," | sed 's/^ *//' | cut -f3- -d" "`" 2 "|"
    append_tabbed "Users:`uptime | sed -e 's/[[:blank:]]users.*$//' -e 's/^.*[[:blank:]]//'`" 2
    append_tabbed "`awk '{print "Load average:" $1 " " $2 " " $3}' < /proc/loadavg`" 2
    endwin 

    col_right
    move_up

    #memory usage
    window "Memory usage" "blue" "20%"
    append_tabbed `cat /proc/meminfo | awk '/MemTotal/ {print "Total:" $2/1024}'` 2
    append_tabbed `cat /proc/meminfo | awk '/MemFree/ {print "Free:" $2/1024}'` 2
    endwin

    col_right
    move_up

    #5 more used process ordered by cpu and memory usage
    window "Processus taking memory and CPU" "green" "50%"
    for i in `seq 2 6`; do
        append_tabbed "`ps ax -o pid,rss,pcpu,ucmd --sort=-cpu,-rss | sed -n "$i,$i p" | awk '{printf "%s: %smo:  %s%%" , $4, $2/1024, $3 }'`" 3
    done
    endwin


    cw_version=`dpkg -l | grep wildix-cw | awk '{print $3}'`
    cw_md5sum=`md5sum /usr/sbin/callweaver | cut -f1 -d" "`
    ver_str="$cw_version   {$cw_md5sum}"

    apid_str="`ps ax | grep 'callweaver .* -v -v -v' | grep -v grep | sed -e 's/^[[:blank:]]*//' -e 's/[[:blank:]].*$//' `"
    if [ "$apid_str" = "" ] ; then 
        echo "Callweaver process not exist in memory.. exiting!!" 1>&2
        exit 1
    fi

    uptime_str="`callweaver -rx 'core show uptime' | grep System | cut -f2 -d":"`"
    achans_str="`callweaver -rx 'core show channels count' | grep channels | sed -e 's/[[:blank:]].*$//'`"
    acalls_str="`callweaver -rx 'core show channels count' | grep capacity`"
    tcalls_str="`callweaver -rx 'core show channels count' | grep processed | sed -e 's/[[:blank:]].*$//'`"
    amem_raw=`cat /proc/$a_pid/status | grep ^VmRSS | sed -e 's/^[^[:blank:]]*[[:blank:]]*//' -e 's/[[:blank:]].*$//' -e "s/\n+//g" `
    amem_str=`expr $amem_raw / 1024`
    amem_reminder_str=`expr $amem_raw % 1024`

    echo "Version                 : $ver_str"    >  $cwstat
    echo "Uptime                  :$uptime_str"  >> $cwstat
    echo "Actual pid              : $apid_str"   >> $cwstat
    echo "Active channels         : $achans_str" >> $cwstat
    echo "Active calls info       : $acalls_str" >> $cwstat
    echo "Total calls processed   : $tcalls_str" >> $cwstat
    echo "Last memory utilization : $amem_str.$amem_reminder_str M" >> $cwstat
    echo "Last observed time      : `date +%T`"  >> $cwstat

    #basic cw statistics
    set_position 0 10
    window "Callweaver " "yellow"
    append_tabbed "Version               : $ver_str" 3
    append_tabbed "Uptime                : $uptime_str" 3
    append_tabbed "Actual pid            : $apid_str" 3
    append_tabbed "Active channels       : $achans_str" 3
    append_tabbed "Active calls info     : $acalls_str" 3
    append_tabbed "Total calls processed : $tcalls_str" 3
    addsep
    if [ $log_updated -eq "1" ] ; then
        rm -f $cwlog.temp
        cat $cwlog | tail -n $log_lines > $cwlog.temp
        log_updated=0
    fi
    append_file $cwlog.temp
    endwin

}
main_loop 0.5
